"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WsOutboundTransporter = void 0;
const AgentConfig_1 = require("../agent/AgentConfig");
const AriesFrameworkError_1 = require("../error/AriesFrameworkError");
class WsOutboundTransporter {
    constructor() {
        this.transportTable = new Map();
        this.supportedSchemes = ['ws', 'wss'];
        // NOTE: Because this method is passed to the event handler this must be a lambda method
        // so 'this' is scoped to the 'WsOutboundTransporter' class instance
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        this.handleMessageEvent = (event) => {
            this.logger.debug('WebSocket message event received.', { url: event.target.url, data: event.data });
            this.agent.receiveMessage(JSON.parse(event.data));
        };
    }
    async start(agent) {
        this.agent = agent;
        const agentConfig = agent.injectionContainer.resolve(AgentConfig_1.AgentConfig);
        this.logger = agentConfig.logger;
        this.logger.debug('Starting WS outbound transport');
        this.WebSocketClass = agentConfig.agentDependencies.WebSocketClass;
    }
    async stop() {
        this.logger.debug('Stopping WS outbound transport');
        this.transportTable.forEach((socket) => {
            socket.removeEventListener('message', this.handleMessageEvent);
            socket.close();
        });
    }
    async sendMessage(outboundPackage) {
        const { connection, payload, endpoint } = outboundPackage;
        this.logger.debug(`Sending outbound message to connection ${connection.id}  (${connection.theirLabel}) over websocket transport.`, payload);
        const isNewSocket = this.hasOpenSocket(connection.id);
        const socket = await this.resolveSocket(connection, endpoint);
        socket.send(JSON.stringify(payload));
        // If the socket was created for this message and we don't have return routing enabled
        // We can close the socket as it shouldn't return messages anymore
        if (isNewSocket && !outboundPackage.responseRequested) {
            socket.close();
        }
    }
    hasOpenSocket(socketId) {
        return this.transportTable.get(socketId) !== undefined;
    }
    async resolveSocket(connection, endpoint) {
        const socketId = connection.id;
        // If we already have a socket connection use it
        let socket = this.transportTable.get(socketId);
        if (!socket) {
            if (!endpoint) {
                throw new AriesFrameworkError_1.AriesFrameworkError(`Missing endpoint. I don't know how and where to send the message.`);
            }
            socket = await this.createSocketConnection(endpoint, socketId);
            this.transportTable.set(socketId, socket);
            this.listenOnWebSocketMessages(socket);
        }
        if (socket.readyState !== this.WebSocketClass.OPEN) {
            throw new AriesFrameworkError_1.AriesFrameworkError('Socket is not open.');
        }
        return socket;
    }
    listenOnWebSocketMessages(socket) {
        socket.addEventListener('message', this.handleMessageEvent);
    }
    createSocketConnection(endpoint, socketId) {
        return new Promise((resolve, reject) => {
            this.logger.debug(`Connecting to WebSocket ${endpoint}`);
            const socket = new this.WebSocketClass(endpoint);
            socket.onopen = () => {
                this.logger.debug(`Successfully connected to WebSocket ${endpoint}`);
                resolve(socket);
            };
            socket.onerror = (error) => {
                this.logger.debug(`Error while connecting to WebSocket ${endpoint}`, {
                    error,
                });
                reject(error);
            };
            socket.onclose = () => {
                socket.removeEventListener('message', this.handleMessageEvent);
                this.transportTable.delete(socketId);
            };
        });
    }
}
exports.WsOutboundTransporter = WsOutboundTransporter;
//# sourceMappingURL=WsOutboundTransporter.js.map